<?php

/**
 * @package   FL Yandex Zen Plugin
 * @author    Дмитрий Васюков https://fictionlabs.ru
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 */

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

use Joomla\Utilities\ArrayHelper;


class plgAjaxFLYandexZen extends JPlugin 
{
	// Construct

	public function __construct($subject, $config)
	{
		jimport('joomla.filesystem.folder');

		parent::__construct($subject, $config);
		$this->loadLanguage();
 
        $this->app 		= JFactory::getApplication();
		$this->input 	= $this->app->input; 
		$this->document = JFactory::getDocument();
	}

	// Get Rss

	public function onAjaxFLYandexZen()
	{	
		// Check Code
		if ($this->input->getString('code') != $this->params->get('channel_code')) {
			return;
		}

		// Set Encoding
		$this->document->setMimeEncoding('application/xml');
    	$this->document->setType('xml');

		// Check Cache
		if ($this->params->get('enable_cache', 0)) {
			// Get Request
			$component 	= $this->input->getString('component', 'all');

			$cacheId 	= md5(serialize($component));
			$cache 		= JFactory::getCache('plg_flyandexzen', '');

			$cache->setCaching(true);
			$cache->setLifeTime($this->params->get('cache_time', JFactory::getConfig()->get('cachetime', 1140)));

			$html 	= $cache->get($cacheId, 'plg_flyandexzen');

			if (!$html) {
				$html = $this->getRss();

				$cache->store($html, $cacheId, 'plg_flyandexzen');
			}
		} else {
			$html = $this->getRss();
		}

		return $html;
	}

	// Get Rss Function

	public function getRss()
	{	
		// Get Request
		$component 	= $this->input->getString('component', 'all');

		// Get Path
		$path 		= JPluginHelper::getLayoutPath('ajax', 'flyandexzen');
		$plgPath    = dirname(__FILE__).'/plugins';

		$items = array();

		// Check All Components
		if ($component == 'all') {
			$plgPath    = dirname(__FILE__).'/plugins';
			if (is_dir($plgPath)) {
				if (is_dir($plgPath)) {
					foreach (JFolder::files($plgPath, '^([-_A-Za-z0-9]+)\.php$') as $file) {
						$plugin 	= basename($file, '.php');
						$filePath	= $plgPath.'/'.$file;
						$plgName	= str_replace('com_', '', $plugin);
						$class  	= 'FLYandexZenCore'.ucfirst($plgName);
						$plgOptions = $this->params->get($plgName.'_options');

						if (is_file($filePath) && !class_exists($class) && $plgOptions->enable) {
							include $filePath;

							$obj = new $class($this->params);

							if (method_exists($obj, 'getContent')) {
								$content 	= $obj->getContent();
								$items 		= array_merge($items, $content);
							}
						}
					}
				}
			}
		} else {
			$component 	= strtolower($component);
			$filePath	= $plgPath.'/com_'.$component.'.php';
			$class  	= 'FLYandexZenCore'.ucfirst($component);
			$plgOptions = $this->params->get($component.'_options');


			
			if (is_file($filePath) && !class_exists($class) && $plgOptions->enable) {
				include $filePath;

				$obj = new $class($this->params);

				if (method_exists($obj, 'getContent')) {
					$content 	= $obj->getContent();
					$items 		= array_merge($items, $content);
				}
			}

		}

		// Render Template
		ob_start();
		include $path;
		$html = ob_get_clean();

		return $html;
	}

	// Render Module By Position

	public function renderModule($position)
	{	
		$moduleHtml 		= array();
		$modules  			= JModuleHelper::getModules($position);
		$attribs  			= array();
		$attribs['style'] 	= 'xhtml';

		if (!empty($modules)) { // Check Empty Modules
			foreach ($modules as $module) {
			    $moduleHtml[] = JModuleHelper::renderModule($module, $attribs);
			}

			// Strip Tags
	        $core 			= new FLYandexZenCore($this->params);
	        $tags 			= $core->getAllowTags($this->params->get('items_tags'));
	        $moduleHtml  	= implode("\n", $moduleHtml);
	        $moduleHtml  	= strip_tags($moduleHtml, $tags);

	        if ($this->params->get('enable_plugins', 0)) {
	        	$moduleHtml     = JHtml::_('content.prepare', $moduleHtml); // Trigger Content Plugins
	        }

	        return $moduleHtml;
		}
        
        return;
        
	}

	// Get Mime Type

	public function getMimeType($filename) {

        $mime_types = array(

            'txt' 	=> 'text/plain',
            'htm' 	=> 'text/html',
            'html' 	=> 'text/html',
            'php' 	=> 'text/html',
            'css' 	=> 'text/css',
            'js' 	=> 'application/javascript',
            'json' 	=> 'application/json',
            'xml' 	=> 'application/xml',
            'swf' 	=> 'application/x-shockwave-flash',
            'flv' 	=> 'video/x-flv',

            // images
            'png' 	=> 'image/png',
            'jpe'	=> 'image/jpeg',
            'jpeg' 	=> 'image/jpeg',
            'jpg' 	=> 'image/jpeg',
            'gif' 	=> 'image/gif',
            'bmp' 	=> 'image/bmp',
            'ico' 	=> 'image/vnd.microsoft.icon',
            'tiff' 	=> 'image/tiff',
            'tif' 	=> 'image/tiff',
            'svg' 	=> 'image/svg+xml',
            'svgz' 	=> 'image/svg+xml',

            // audio/video
            'mp3' 	=> 'audio/mpeg',
            'qt' 	=> 'video/quicktime',
            'mov' 	=> 'video/quicktime',
            'mp4' 	=> 'video/mp4',
           	'avi' 	=> 'video/x-msvideo',
           	'wmv' 	=> 'video/x-ms-wmv'
        );

        $explode 	= explode('.', $filename);
        $ext 		= strtolower(array_pop($explode));

        if (array_key_exists($ext, $mime_types)) {
            return $mime_types[$ext];
        } 

        // if (function_exists('finfo_open')) {
        //     $finfo = finfo_open(FILEINFO_MIME);
        //     $mimetype = finfo_file($finfo, $filename);
        //     finfo_close($finfo);
        //     return $mimetype;
        // }

        return 'application/octet-stream';
    }
}

/**
 * @package   FL Yandex Zen Plugin
 * @author    Дмитрий Васюков https://fictionlabs.ru
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 */

class FLYandexZenCore
{	
	// Construct

	function __construct($params)
	{
		$this->params   = $params;
		$this->app 		= JFactory::getApplication();
        $this->db 		= JFactory::getDbo();
		$this->input 	= $this->app->input; 
		$this->tags  	= '<p><span><img><figure><figcaption>';
		$this->ssl 		= (JFactory::getConfig()->get('force_ssl', 0) == 2) ? 1 : -1;
	}

	// Get Items

	public function getItems()
	{
		return array();
	}

	// Get Content

	public function getContent()
	{
		return array();
	}

	// Get Item Author

	public function getAuthor($id)
	{
		if ($this->params->get('items_author')) {
			$user = JFactory::getUser($id);
			$userName = $user->name;
		} else {
			$userName = $this->params->get('items_author_name', 'Администратор');
		}

		return $userName;
	}

	// Prepare Content Fucntion

	public function prepareContent($html)
	{	
		if ($this->params->get('enable_plugins', 0)) {
			$html 	= JHtml::_('content.prepare', $html); // Trigger Content Plugins
		}

		$allow_attr = array('href', 'src', 'http-equiv', 'content', 'charset');
		$allow_tags = array('br', 'img', 'script', 'meta');

		// Remove scripts

		$html = preg_replace('#<script(.*?)>(.*?)</script>#is', '', $html);

		$dom = new DOMDocument('1.0','UTF-8');
		$dom->encoding = "UTF-8";
		$dom->preserveWhiteSpace = false;
		libxml_use_internal_errors(true);
		$dom->loadHTML('<meta http-equiv="Content-Type" content="text/html; charset=utf-8">'.mb_convert_encoding($html, 'HTML-ENTITIES', 'UTF-8'));

		$xpath = new DOMXPath($dom);
		
		// Remove Tags With Noflyandexzen Class

		$removeElements = $xpath->query("//*[contains(@class, 'noflyandexzen')]");

		if ($removeElements) {
			foreach ($removeElements as $node) {
			    $node->parentNode->removeChild($node);
			}
		}

		// Clean All Tags

		foreach ($xpath->query('//@*') as $node) {
		    if(!in_array($node->nodeName, $allow_attr)) {
		        $node->parentNode->removeAttribute($node->nodeName);
		    }		
		}

		// Remove Empty Tags

		foreach ($xpath->query('//*[not(node())]') as $node) {
			if (!in_array($node->nodeName, $allow_tags) && !strlen(trim($node->textContent))) {
			    $node->parentNode->removeChild($node);
			}
		}

		// Save Original Html

		$result = preg_replace('/^<!DOCTYPE.+?>/', '', str_replace( array('<html>', '</html>', '<body>', '</body>', '<head>', '</head>', '<meta http-equiv="Content-Type" content="text/html; charset=utf-8">'), array('', '', '', '', '', '', ''), $dom->saveHTML()));

		// Strip tags
		
		$result = strip_tags($result, $this->tags);

		// Remove Spaces

		$result = trim(preg_replace('/\s+/',' ',$result));

		return htmlspecialchars_decode($result);
	}

	// Clear Text

	public function clearText($text)
	{
        return trim(addslashes(htmlspecialchars(strip_tags($text))));
    }

    // Get Images From String

	public function getImagesFromHtml($html)
	{
		$images = array();

		preg_match_all('/< *img[^>]*src *= *["\']?([^"\']*)/i', $html, $matches);

		if (!empty($matches)) {
			$images = $matches[1];
		}

		return $images; 
	}

	// Get Video From String

	public function getVideoFromHtml($html) 
	{
		$video = array();

		preg_match_all('/< *iframe[^>]*src *= *["\']?([^"\']*)/i', $html, $matches);

		if (!empty($matches)) {
			$video = $matches[1];
		}

		return $video; 
	}

	// Get Images From Content

	public function getImagesFromJson($string) 
	{
		$images 	= array();
		$json 		= json_decode($string);

		if (!empty($json->image_fulltext)) {
			$images[] = JURI::root().$json->image_fulltext;
		}

		// if (!empty($json->image_intro)) {
		// 	$images[] = JURI::root().$json->image_intro;
		// }

		return $images; 
	}

	// Get Specific Categories From Categories List

	public function getCategories($categories = array()) 
	{
		$cats = array();

		$categoriesAllow = array('Авто', 'Война', 'Дизайн', 'Дом', 'Еда', 'Здоровье', 'Знаменитости', 'Игры', 'Кино', 'Культура', 'Литература', 'Мода', 'Музыка', 'Наука', 'Общество', 'Политика', 'Природа', 'Происшествия', 'Психология', 'Путешествия', 'Спорт', 'Технологии', 'Фотографии', 'Хобби', 'Экономика', 'Юмор');

		$cats = array_intersect($categoriesAllow, $categories);

		return $cats;
	}

	// Extra Image Layout

	public function getImageLayout($src, $title) 
	{
		$html = '<figure><img src="'.$src.'"><figcaption>'.$title.'</figcaption></figure>';

		return $html;
	}
}