<?php
/**
 * @package   FL Yandex Zen Plugin
 * @author    Дмитрий Васюков http://fictionlabs.ru
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 */

defined('_JEXEC') or die;

class FLYandexZenCoreK2 extends FLYandexZenCore {

	// Get COM_K2 Content

	public function getContent()
	{	
		// make sure K2 exist
		jimport('joomla.filesystem.file');
		if (!JFile::exists(JPATH_ADMINISTRATOR.'/components/com_k2/k2.php')
				|| !JComponentHelper::getComponent('com_k2', true)->enabled) {
			return;
		}

		require_once(JPATH_SITE.'/components/com_k2/helpers/route.php');
		require_once(JPATH_SITE.'/components/com_k2/helpers/utilities.php');

		$result 	= array();
		$items 		= $this->getItems();

		if(!empty($items)) {
			foreach ($items as $item) {	
				$date 		   	= new JDate($item->created);
				$link 			= JRoute::_(K2HelperRoute::getItemRoute($item->id.':'.urlencode($item->alias), $item->catid.':'.urlencode($item->categoryalias)), false, $this->ssl);
				$image 			= $this->getImage($item->id);
				$content 		= $item->introtext.$item->fulltext;

				if (!empty($image)) {
					$content 	= $this->getImageLayout($image[0], $item->title).$content;
				}
				
				$result[] = array(
					'title' 		=> htmlspecialchars($item->title),
					'images' 		=> $this->getImagesFromHtml($content),
					'video' 		=> $this->getVideoFromHtml($content),
					'categories'    => $this->getCategories(array($item->categoryname)),
					'link' 			=> $link,
					'date' 			=> $date->toRFC822(true),
					'author' 		=> $this->getAuthor($item->created_by),
					'content' 		=> $this->prepareContent($content),
				);
			}
		}

		return $result;
	}

	// Get Content Items

	public function getItems()
	{
		$items 			= array();
		$options 		= $this->params->get('k2_options');
		$catFilter 		= $options->k2_cat_filter;
		$limit 			= isset($options->k2_count) ? (int) $options->k2_count : 500;
		$catId 			= isset($options->k2_catid) ? $options->k2_catid : '';
		$getChildren 	= $options->k2_get_children;
		$featured 		= $options->k2_show_featured;
		$image 			= $options->k2_add_image;

		$jnow 			= JFactory::getDate();
		$now 			= $jnow->toSql();
		$nullDate 		= $this->db->getNullDate();

		$query 			= "SELECT DISTINCT i.*,";
		$query 			.= "c.name AS categoryname,c.id AS categoryid, c.alias AS categoryalias, c.params AS categoryparams";
		$query 			.= " FROM #__k2_items as i RIGHT JOIN #__k2_categories c ON c.id = i.catid";
		$query 	 		.= " WHERE i.published = 1 AND i.trash = 0 AND c.published = 1";
		$query 			.= " AND ( i.publish_up = ".$this->db->Quote($nullDate)." OR i.publish_up <= ".$this->db->Quote($now)." )";
		$query 			.= " AND ( i.publish_down = ".$this->db->Quote($nullDate)." OR i.publish_down >= ".$this->db->Quote($now)." )";


		if ($catFilter) {

			if (!is_null($catId)) {

				if (is_array($catId)) {

					if ($getChildren) {
						$itemListModel = K2Model::getInstance('Itemlist', 'K2Model');
						$categories = $itemListModel->getCategoryTree($catId);
						$sql = @implode(',', $categories);
						$query .= " AND i.catid IN ({$sql})";
					} else {
						JArrayHelper::toInteger($cid);
						$query .= " AND i.catid IN(".implode(',', $catId).")";
					}
				} else {
					if ($getChildren) {
						$itemListModel = K2Model::getInstance('Itemlist', 'K2Model');
						$categories = $itemListModel->getCategoryTree($catId);
						$sql = @implode(',', $categories);
						$query .= " AND i.catid IN ({$sql})";
					} else {
						$query .= " AND i.catid=".(int)$catId;
					}
				}
			}
		}
			
		if ($featured == '0') {
			$query .= " AND i.featured != 1";
		}

		if ($featured == '1') {
			$query .= " AND i.featured = 1";
		}

		if ($this->params->get('enable_random')) {
			$query .= " ORDER BY RAND()";
		} else {
			$query .= " ORDER BY i.created DESC";
		}

		$this->db->setQuery($query);
		$items = $this->db->loadObjectList();

		return $items;
	}

	// Get K2 Image Function

	public function getImage($id)
	{
		$image = array();

		if (JFile::exists(JPATH_SITE.'/media/k2/items/cache/'.md5("Image".$id).'_XL.jpg')) {
			$image[] = JURI::root().'media/k2/items/cache/'.md5("Image".$id).'_XL.jpg';
		}

		return $image;
	}
}